//@line 36 "/home/cltbld/build/maemo-1.9.2-nightly/mozilla-1.9.2/mobile/locales/en-US/mobile-l10n.js"

//@line 38 "/home/cltbld/build/maemo-1.9.2-nightly/mozilla-1.9.2/mobile/locales/en-US/mobile-l10n.js"

pref("general.useragent.locale", "en-US");
